@@ -11,17 +11,18 @@
 # References:   1) http://www.cybercom.net/~dcoffin/dcraw/
 #               2) http://www.wonderland.org/crw/
 #               3) http://xyrion.org/ciff/CIFFspecV1R04.pdf
+#               4) Dave Nicholson private communication (PowerShot S30)
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::CanonRaw;
 
 use strict;
 use vars qw($VERSION $AUTOLOAD %crwTagFormat);
-use Image::ExifTool qw(:DataAccess);
+use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::Canon;
 
-$VERSION = '1.35';
+$VERSION = '1.49';
 
 sub WriteCRW($$);
 sub ProcessCanonRaw($$$);
@@ -52,11 +53,11 @@ sub BuildMakerNotes($$$$$$);
     WRITE_PROC => \&WriteCanonRaw,
     CHECK_PROC => \&CheckCanonRaw,
     WRITABLE => 1,
-    0x0000 => 'NullRecord', #3
+    0x0000 => { Name => 'NullRecord', Writable => 'undef' }, #3
     0x0001 => { #3
         Name => 'FreeBytes',
         Format => 'undef',
-        ValueConv => '\$val',
+        Binary => 1,
     },
     0x0032 => { Name => 'CanonColorInfo1', Writable => 0 },
     0x0805 => [
@@ -64,9 +65,11 @@ sub BuildMakerNotes($$$$$$);
         {
             Condition => '$self->{DIR_NAME} eq "ImageDescription"',
             Name => 'CanonFileDescription',
+            Writable => 'string[32]',
         },
         {
             Name => 'UserComment',
+            Writable => 'string[256]',
         },
     ],
     0x080a => {
@@ -76,18 +79,16 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::CanonRaw::MakeModel',
         },
     },
-    0x080b => 'CanonFirmwareVersion',
-    0x080c => 'ComponentVersion', #3
-    0x080d => 'ROMOperationMode', #3
-    0x0810 => {
-        Name => 'OwnerName',
-        Description => "Owner's Name",
-    },
-    0x0815 => 'CanonImageType',
-    0x0816 => 'OriginalFileName',
-    0x0817 => 'ThumbnailFileName',
+    0x080b => { Name => 'CanonFirmwareVersion', Writable => 'string[32]' },
+    0x080c => { Name => 'ComponentVersion',     Writable => 'string'     }, #3
+    0x080d => { Name => 'ROMOperationMode',     Writable => 'string[8]'  }, #3
+    0x0810 => { Name => 'OwnerName',            Writable => 'string[32]' },
+    0x0815 => { Name => 'CanonImageType',       Writable => 'string[32]' },
+    0x0816 => { Name => 'OriginalFileName',     Writable => 'string[32]' },
+    0x0817 => { Name => 'ThumbnailFileName',    Writable => 'string[32]' },
     0x100a => { #3
         Name => 'TargetImageType',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Real-world Subject',
             1 => 'Written Document',
@@ -95,6 +96,7 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1010 => { #3
         Name => 'ShutterReleaseMethod',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Single Shot',
             2 => 'Continuous Shooting',
@@ -102,13 +104,14 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1011 => { #3
         Name => 'ShutterReleaseTiming',
+        Writable => 'int16u',
         PrintConv => {
             0 => 'Priority on shutter',
             1 => 'Priority on focus',
         },
     },
-    0x1016 => 'ReleaseSetting', #3
-    0x101c => 'BaseISO', #3
+    0x1016 => { Name => 'ReleaseSetting',       Writable => 'int16u' }, #3
+    0x101c => { Name => 'BaseISO',              Writable => 'int16u' }, #3
     0x1028=> { #PH
         Name => 'CanonFlashInfo',
         Writable => 'int16u',
@@ -129,7 +132,14 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::Canon::ShotInfo',
         },
     },
-    0x102c => { Name => 'CanonColorInfo2', Writable => 0 },
+    0x102c => {
+        Name => 'CanonColorInfo2',
+        Writable => 0,
+        # for the S30, the following information has been decoded: (ref 4)
+        # offset 66: int32u - shutter half press time in ms
+        # offset 70: int32u - image capture time in ms
+        # offset 74: int16u - custom white balance flag (0=Off, 512=On)
+    },
     0x102d => {
         Name => 'CanonCameraSettings',
         Writable => 0,
@@ -137,6 +147,14 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::Canon::CameraSettings',
         },
     },
+    0x1030 => { #4
+        Name => 'WhiteSample',
+        Writable => 0,
+        SubDirectory => {
+            Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
+            TagTable => 'Image::ExifTool::CanonRaw::WhiteSample',
+        },
+    },
     0x1031 => {
         Name => 'SensorInfo',
         Writable => 0,
@@ -144,39 +162,48 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::Canon::SensorInfo',
         },
     },
-    # this tag has only be verified for the 10D in CRW files, but this
-    # is the way it works for the other models with JPG files...
+    # this tag has only be verified for the 10D in CRW files, but the D30 and D60
+    # also produce CRW images and have CustomFunction information in their JPEG's
     0x1033 => [
         {
-            Condition => '$self->{CameraModel} =~ /10D/',
-            Name => 'CanonCustomFunctions10D',
-            Writable => 0,
+            Name => 'CustomFunctions10D',
+            Condition => '$self->{Model} =~ /EOS 10D/',
             SubDirectory => {
+                Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
                 TagTable => 'Image::ExifTool::CanonCustom::Functions10D',
             },
         },
         {
-            Condition => '$self->{CameraModel} =~ /20D/',
-            Name => 'CanonCustomFunctions20D',
-            Writable => 0,
+            Name => 'CustomFunctionsD30',
+            Condition => '$self->{Model} =~ /EOS D30\b/',
             SubDirectory => {
-                TagTable => 'Image::ExifTool::CanonCustom::Functions20D',
+                Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
+                TagTable => 'Image::ExifTool::CanonCustom::FunctionsD30',
             },
         },
         {
-            # assume everything else is a D30/D60
-            Name => 'CanonCustomFunctionsD30',
-            Writable => 0,
+            Name => 'CustomFunctionsD60',
+            Condition => '$self->{Model} =~ /EOS D60\b/',
             SubDirectory => {
+                # the stored size in the D60 apparently doesn't include the size word:
+                Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size-2,$size)',
+                # (D60 custom functions are basically the same as D30)
                 TagTable => 'Image::ExifTool::CanonCustom::FunctionsD30',
             },
         },
+        {
+            Name => 'CustomFunctionsUnknown',
+            SubDirectory => {
+                Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
+                TagTable => 'Image::ExifTool::CanonCustom::FuncsUnknown',
+            },
+        },
     ],
     0x1038 => {
-        Name => 'CanonPictureInfo',
+        Name => 'CanonAFInfo',
         Writable => 0,
         SubDirectory => {
-            TagTable => 'Image::ExifTool::Canon::PictureInfo',
+            TagTable => 'Image::ExifTool::Canon::AFInfo',
         },
     },
     0x1093 => {
@@ -203,9 +230,13 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::CanonRaw::RawJpgInfo',
         },
     },
-    0x10ae => 'ColorTemperature',
+    0x10ae => {
+        Name => 'ColorTemperature',
+        Writable => 'int16u',
+    },
     0x10b4 => {
         Name => 'ColorSpace',
+        Writable => 'int16u',
         PrintConv => {
             1 => 'sRGB',
             2 => 'Adobe RGB',
@@ -219,26 +250,46 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::CanonRaw::ImageFormat',
         },
     },
-    0x1804 => 'RecordID', #3
+    0x1804 => { Name => 'RecordID', Writable => 'int32u' }, #3
     0x1806 => { #3
         Name => 'SelfTimerTime',
+        Writable => 'int32u',
         ValueConv => '$val / 1000',
         ValueConvInv => '$val * 1000',
-        PrintConv => '"$val sec"',
-        PrintConvInv => '$val=~s/\s*sec.*//;$val',
+        PrintConv => '"$val s"',
+        PrintConvInv => '$val=~s/\s*s.*//;$val',
     },
     0x1807 => {
         Name => 'TargetDistanceSetting',
         Format => 'float',
         PrintConv => '"$val mm"',
-        PrintConvInv => '$val=~s/\s*mm//;$val',
-    },
-    0x180b => {
-        Name => 'SerialNumber',
-        Description => 'Camera Body No.',
-        PrintConv => 'sprintf("%.10d",$val)',
-        PrintConvInv => '$val',
+        PrintConvInv => '$val=~s/\s*mm$//;$val',
     },
+    0x180b => [
+        {
+            # D30
+            Name => 'SerialNumber',
+            Description => 'Camera Body No.',
+            Condition => '$$self{Model} =~ /EOS D30\b/',
+            Writable => 'int32u',
+            PrintConv => 'sprintf("%x-%.5d",$val>>16,$val&0xffff)',
+            PrintConvInv => '$val=~/(.*)-(\d+)/ ? (hex($1)<<16)+$2 : undef',
+        },
+        {
+            # all EOS models (D30, 10D, 300D)
+            Name => 'SerialNumber',
+            Description => 'Camera Body No.',
+            Condition => '$$self{Model} =~ /EOS/',
+            Writable => 'int32u',
+            PrintConv => 'sprintf("%.10d",$val)',
+            PrintConvInv => '$val',
+        },
+        {
+            # this is not SerialNumber for PowerShot models (but what is it?) - PH
+            Name => 'UnknownNumber',
+            Unknown => 1,
+        },
+    ],
     0x180e => {
         Name => 'TimeStamp',
         Writable => 0,
@@ -262,10 +313,15 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1814 => { #3
         Name => 'MeasuredEV',
+        Notes => q{
+            this the Canon name for what could better be called MeasuredLV, and is
+            offset by about -5 EV from the calculated LV for most models
+        },
         Format => 'float',
     },
     0x1817 => {
         Name => 'FileNumber',
+        Writable => 'int32u',
         Groups => { 2 => 'Image' },
         PrintConv => '$_=$val;s/(\d+)(\d{4})/$1-$2/;$_',
         PrintConvInv => '$_=$val;s/-//;$_',
@@ -279,17 +335,21 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1834 => { #PH
         Name => 'CanonModelID',
-        Writable => 0,
+        Writable => 'int32u',
         PrintHex => 1,
         Notes => q{
             this is the complete list of model ID numbers, but note that many of these
             models do not produce CRW images
         },
+        SeparateTable => 'Canon CanonModelID',
         PrintConv => \%Image::ExifTool::Canon::canonModelID,
     },
     0x1835 => {
         Name => 'DecoderTable',
         Writable => 0,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::CanonRaw::DecoderTable',
+        },
     },
     0x183b => { #PH
         # display format for serial number
@@ -304,22 +364,20 @@ sub BuildMakerNotes($$$$$$);
     0x2005 => {
         Name => 'RawData',
         Writable => 0,
-        ValueConv => '\$val',
+        Binary => 1,
     },
     0x2007 => {
         Name => 'JpgFromRaw',
         Writable => 'resize',  # 'resize' allows this value to change size
         Permanent => 0,
-        ValueConv => '$self->ValidateImage(\$val,$tag)',
-        ValueConvInv => '$val',
+        RawConv => '$self->ValidateImage(\$val,$tag)',
     },
     0x2008 => {
         Name => 'ThumbnailImage',
         Writable => 'resize',  # 'resize' allows this value to change size
         WriteCheck => '$self->CheckImage(\$val)',
         Permanent => 0,
-        ValueConv => '$self->ValidateImage(\$val,$tag)',
-        ValueConvInv => '$val',
+        RawConv => '$self->ValidateImage(\$val,$tag)',
     },
     # the following entries are subdirectories
     # (any 0x28 and 0x30 tag types are handled automatically by the decoding logic)
@@ -365,6 +423,7 @@ sub BuildMakerNotes($$$$$$);
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    DATAMEMBER => [ 0, 6 ], # indices of data members to extract when writing
     WRITABLE => 1,
     FORMAT => 'string',
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
@@ -373,13 +432,15 @@ sub BuildMakerNotes($$$$$$);
     0 => {
         Name => 'Make',
         Format => 'string[6]',  # "Canon\0"
-        RawConv => '$self->{CameraMake} = $val',
+        DataMember => 'Make',
+        RawConv => '$self->{Make} = $val',
     },
     6 => {
         Name => 'Model',
-        Format => 'string[$size-6]',
+        Format => 'string', # no size = to the end of the data
         Description => 'Camera Model Name',
-        RawConv => '$self->{CameraModel} = $val',
+        DataMember => 'Model',
+        RawConv => '$self->{Model} = $val',
     },
 );
 
@@ -398,7 +459,7 @@ sub BuildMakerNotes($$$$$$);
         ValueConv => 'ConvertUnixTime($val)',
         ValueConvInv => 'GetUnixTime($val)',
         PrintConv => '$self->ConvertDateTime($val)',
-        PrintConvInv => '$val',
+        PrintConvInv => '$self->InverseDateTime($val)',
     },
     1 => { #3
         Name => 'TimeZoneCode',
@@ -487,8 +548,21 @@ sub BuildMakerNotes($$$$$$);
     FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
     0 => 'ExposureCompensation',
-    1 => 'TvValue',
-    2 => 'AvValue',
+    1 => {
+        Name => 'ShutterSpeedValue',
+        ValueConv => 'abs($val)<100 ? 1/(2**$val) : 0',
+        ValueConvInv => '$val>0 ? -log($val)/log(2) : -100',
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+        # do eval to convert things like '1/100'
+        PrintConvInv => 'eval $val',
+    },
+    2 => {
+        Name => 'ApertureValue',
+        ValueConv => '2 ** ($val / 2)',
+        ValueConvInv => '$val>0 ? 2*log($val)/log(2) : 0',
+        PrintConv => 'sprintf("%.1f",$val)',
+        PrintConvInv => '$val',
+    },
 );
 
 %Image::ExifTool::CanonRaw::ImageInfo = (
@@ -496,7 +570,7 @@ sub BuildMakerNotes($$$$$$);
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
     FORMAT => 'int32u',
-    FIRST_ENTRY => 1,
+    FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
     # Note: Don't make these writable (except rotation) because it confuses
     # Canon decoding software if the are changed
@@ -516,6 +590,35 @@ sub BuildMakerNotes($$$$$$);
     6 => 'ColorBW', #3
 );
 
+# ref 4
+%Image::ExifTool::CanonRaw::DecoderTable = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FORMAT => 'int32u',
+    FIRST_ENTRY => 0,
+    0 => 'DecoderTableNumber',
+    2 => 'CompressedDataOffset',
+    3 => 'CompressedDataLength',
+);
+
+# ref 1/4
+%Image::ExifTool::CanonRaw::WhiteSample = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FORMAT => 'int16u',
+    FIRST_ENTRY => 1,
+    1 => 'WhiteSampleWidth',
+    2 => 'WhiteSampleHeight',
+    3 => 'WhiteSampleLeftBorder',
+    4 => 'WhiteSampleTopBorder',
+    5 => 'WhiteSampleBits',
+    # this is followed by the encrypted white sample values (ref 1)
+);
+
 #------------------------------------------------------------------------------
 # AutoLoad our writer routines when necessary
 #
@@ -622,7 +725,7 @@ sub ProcessCanonRaw($$$)
             } else {
                 $value = "Binary data $size bytes";
                 if ($tagInfo) {
-                    if ($exifTool->Options('Binary')) {
+                    if ($exifTool->Options('Binary') or $verbose) {
                         # read the value anyway
                         unless ($raf->Seek($ptr, 0) and $raf->Read($value, $size) == $size) {
                             $exifTool->Warn(sprintf("Error reading %d bytes from 0x%x",$size,$ptr));
@@ -668,7 +771,7 @@ sub ProcessCanonRaw($$$)
             my $name = $$tagInfo{Name};
             my $newTagTable;
             if ($$subdir{TagTable}) {
-                $newTagTable = Image::ExifTool::GetTagTable($$subdir{TagTable});
+                $newTagTable = GetTagTable($$subdir{TagTable});
                 unless ($newTagTable) {
                     warn "Unknown tag table $$subdir{TagTable}\n";
                     next;
@@ -724,7 +827,7 @@ sub ProcessCRW($$)
     SetByteOrder($buff)           or return 0;
     $raf->Read($buff,4) == 4      or return 0;
     $raf->Read($sig,8) == 8       or return 0;  # get file signature
-    $sig eq 'HEAPCCDR'            or return 0;  # validate signature
+    $sig =~ /^HEAP(CCDR|JPGM)/    or return 0;  # validate signature
     my $hlen = Get32u(\$buff, 0);
 
     $raf->Seek(0, 2)              or return 0;  # seek to end of file
@@ -733,7 +836,8 @@ sub ProcessCRW($$)
     # initialize maker note data if building maker notes
     $buildMakerNotes and InitMakerNotes($exifTool);
 
-    $exifTool->SetFileType();   # set the FileType tag
+    # set the FileType tag unless already done (ie. APP0 CIFF record in JPEG image)
+    $exifTool->SetFileType() unless $exifTool->GetValue('FileType');
 
     # build directory information for main raw directory
     my %dirInfo = (
@@ -746,7 +850,7 @@ sub ProcessCRW($$)
     );
 
     # process the raw directory
-    my $rawTagTable = Image::ExifTool::GetTagTable('Image::ExifTool::CanonRaw::Main');
+    my $rawTagTable = GetTagTable('Image::ExifTool::CanonRaw::Main');
     unless ($exifTool->ProcessDirectory(\%dirInfo, $rawTagTable)) {
         $exifTool->Warn('CRW file format error');
     }
@@ -754,6 +858,12 @@ sub ProcessCRW($$)
     # finish building maker notes if necessary
     $buildMakerNotes and SaveMakerNotes($exifTool);
 
+    # process trailers if they exist in CRW file (not in CIFF information!)
+    if ($$exifTool{FILE_TYPE} eq 'CRW') {
+        my $trailInfo = Image::ExifTool::IdentifyTrailer($raf);
+        $exifTool->ProcessTrailers($trailInfo) if $trailInfo;
+    }
+
     return 1;
 }
 
@@ -784,7 +894,7 @@ tags.)
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -803,6 +913,10 @@ under the same terms as Perl itself.
 
 =back
 
+=head1 ACKNOWLEDGEMENTS
+
+Thanks to Dave Nicholson for decoding a number of new tags.
+
 =head1 SEE ALSO
 
 L<Image::ExifTool::TagNames/CanonRaw Tags>,
