@@ -12,7 +12,7 @@ package Image::ExifTool::KyoceraRaw;
 
 use strict;
 use vars qw($VERSION);
-use Image::ExifTool qw(:DataAccess);
+use Image::ExifTool qw(:DataAccess :Utils);
 
 $VERSION = '1.00';
 
@@ -126,7 +126,7 @@ sub ProcessRAW($$)
         DirStart => 0,
         DirLen => $size,
     );
-    my $tagTablePtr = Image::ExifTool::GetTagTable('Image::ExifTool::KyoceraRaw::Main');
+    my $tagTablePtr = GetTagTable('Image::ExifTool::KyoceraRaw::Main');
     $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
     return 1;
 }
@@ -150,7 +150,7 @@ meta information from Kyocera Contax N Digital RAW images.
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
