@@ -18,31 +18,37 @@
 #              10) http://homepage3.nifty.com/kamisaka/makernote/makernote_km.htm
 #              11) http://www.dyxum.com/dforum/forum_posts.asp?TID=6371&PN=1 and
 #                  http://www.dyxum.com/dAdmin/lenses/MakerNoteList_Public.asp?stro=makr
+#                  http://dyxum.com/dforum/forum_posts.asp?TID=23435&PN=2
 #              12) http://www.minolta-forum.de/forum/index.php?showtopic=14914
 #              13) http://www.mhohner.de/minolta/lenses.php
 #              14) Jeffery Small private communication (tests with 7D)
 #              15) http://homepage3.nifty.com/kamisaka/makernote/makernote_sony.htm
+#              16) Thomas Kassner private communication
+#              17) Mladen Sever private communication
+#              JD) Jens Duttke private communication
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Minolta;
 
 use strict;
-use vars qw($VERSION %minoltaLensIDs %minoltaColorMode);
+use vars qw($VERSION %minoltaLensTypes %minoltaColorMode %sonyColorMode %minoltaSceneMode);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.31';
+$VERSION = '1.50';
 
 # lens ID numbers (ref 3)
-%minoltaLensIDs = (
+%minoltaLensTypes = (
     0 => 'Minolta AF 28-85mm F3.5-4.5',
     1 => 'Minolta AF 80-200mm F2.8 HS-APO G',
     2 => 'Minolta AF 28-70mm F2.8 G',
     3 => 'Minolta AF 28-80mm F4-5.6',
     5 => 'Minolta AF 35-70mm F3.5-4.5',
     6 => 'Minolta AF 24-85mm F3.5-4.5 [New]',
-    # 7 => 'AF 100-400mm F4.5-6.7 (D)', ??
-    7 => 'Minolta AF 100-300mm F4.5-5.6 APO [New]',
+  # 7 => 'AF 100-400mm F4.5-6.7 (D)', ??
+    7 => 'Minolta AF 100-300mm F4.5-5.6 APO [New] or 100-400mm or Sigma Lens',
+    7.1 => 'Minolta AF 100-400mm F4.5-6.7 APO', #JD
+    7.2 => 'Sigma AF 100-300mm F4 EX DG IF', #JD
     8 => 'Minolta AF 70-210mm F4.5-5.6',
     9 => 'Minolta AF 50mm F3.5 Macro',
     10 => 'Minolta AF 28-105mm F3.5-4.5 [New]',
@@ -55,61 +61,133 @@ $VERSION = '1.31';
     17 => 'Minolta AF 20-35mm F3.5-4.5',
     18 => 'Minolta AF 28-80mm F3.5-5.6 II',
     19 => 'Minolta AF 35mm F1.4',
-    20 => 'Minolta STF 135mm F2.8 [T4.5]',
+    20 => 'Minolta/Sony STF 135mm F2.8 [T4.5]',
     22 => 'Minolta AF 35-80mm F4-5.6',
     23 => 'Minolta AF 200mm F4 G APO Macro',
-    24 => 'Minolta AF 24-105mm F3.5-4.5 (D)', # or Sigma 18-50mm F2.8 or Sigma 17-70mm F2.8-4.5 (D)',
-    25 => 'Minolta AF 100-300mm F4.5-5.6 (D)',
+    24 => 'Minolta/Sony AF 24-105mm F3.5-4.5 (D) or Sigma Lens',
+    24.1 => 'Sigma 18-50mm F2.8',
+    24.2 => 'Sigma 17-70mm F2.8-4.5 (D)',
+    24.3 => 'Sigma 20-40mm F2.8 EX DG Aspherical IF', #JD
+    24.4 => 'Tamron SP AF 28-75mm F2.8 XR Di (IF) Macro', #JD
+    25 => 'Minolta AF 100-300mm F4.5-5.6 (APO D) or Sigma Lens',
+    25.1 => 'Sigma 100-300mm F4 EX (APO (D) or D IF)', #JD
+    25.2 => 'Sigma 70mm F2.8 EX DG Macro', #JD
     27 => 'Minolta AF 85mm F1.4 G',
-    28 => 'Minolta AF 100mm F2.8 Macro (D)',
+    28 => 'Minolta AF 100mm F2.8 Macro (D) or Tamron Lens',
+    28.1 => 'Tamron SP AF 90mm F2.8 Di Macro', #JD
     29 => 'Minolta AF 75-300mm F4.5-5.6 (D)',
-    30 => 'Minolta AF 28-80mm F3.5-5.6 (D)', # or Sigma AF 12-24mm F4.5-5.6 EX DG',
-    31 => 'Minolta AF 50mm F2.8 Macro (D) or AF 50mm F3.5 Macro',
-    # 32 => 'AF 100-400mm F4.5-6.7 (D) x1.5', ??
-    32 => 'Minolta AF 300mm F2.8 G',
-    33 => 'Minolta AF 70-200mm F2.8 G (D) SSM',
+    30 => 'Minolta AF 28-80mm F3.5-5.6 (D) or Sigma Lens',
+    30.1 => 'Sigma AF 10-20mm F4-5.6 EX DC', #JD
+    30.2 => 'Sigma AF 12-24mm F4.5-5.6 EX DG',
+    30.3 => 'Sigma 28-70mm EX DG F2.8', #16
+    30.4 => 'Sigma 55-200mm F4-5.6 DC', #JD
+    31 => 'Minolta/Sony AF 50mm F2.8 Macro (D) or F3.5',
+    31.1 => 'Minolta/Sony AF 50mm F3.5 Macro',
+    32 => 'Minolta AF 300mm F2.8 G or 100-400mm F4.5-6.7 x1.5',
+    32.1 => 'Minolta AF 100-400mm F4.5-6.7 (D) x1.5', #?
+    33 => 'Minolta/Sony AF 70-200mm F2.8 G (D) SSM',
     35 => 'Minolta AF 85mm F1.4 G (D) Limited',
     36 => 'Minolta AF 28-100mm F3.5-5.6 (D)',
-    38 => 'KonicaMinolta AF 17-35mm F2.8-4 (D)',
+    38 => 'Minolta AF 17-35mm F2.8-4 (D)',
     39 => 'Minolta AF 28-75mm F2.8 (D)',
-    40 => 'KonicaMinolta or Sony AF DT 18-70mm F3.5-5.6 (D)',
-    41 => 'Minolta AF DT 11-18mm F4.5-5.6 (D)',
+    40 => 'Minolta/Sony AF DT 18-70mm F3.5-5.6 (D)',
+    41 => 'Minolta/Sony AF DT 11-18mm F4.5-5.6 (D) or Tamron Lens',
+    41.1 => 'Tamron SP AF 11-18mm F4.5-5.6 Di II LD Aspherical IF', #JD
     42 => 'Minolta AF DT 18-200mm F3.5-6.3 (D)',
     43 => 'Minolta AF 35mm F1.4 G',
     44 => 'Minolta AF 50mm F1.4',
     45 => 'Carl Zeiss Planar T* 85mm F1.4 ZA',
     46 => 'Carl Zeiss Vario-Sonnar T* DT 16-80mm F3.5-4.5 ZA',
     47 => 'Carl Zeiss Sonnar T* 135mm F1.8 ZA',
-    128 => 'Tamron 18-200, 28-300 or 80-300mm F3.5-6.3',
-    129 => 'Tamron 200-400mm F5.6 LD', #12
+    48 => 'Carl Zeiss Vario-Sonnar T* 24-70mm F2.8 ZA SSM (SAL-2470Z)', #11
+    49 => 'Sony AF DT 55-200mm F4-5.6', #JD
+    50 => 'Sony AF DT 18-250mm F3.5-6.3', #11
+    51 => 'Sony AF DT 16-105mm F3.5-5.6 or 55-200mm F4-5.5', #11
+    51.1 => 'Sony AF DT 55-200mm F4-5.5', #11
+    52 => 'Sony 70-300mm F4.5-5.6 G SSM', #JD
+    53 => 'Sony AF 70-400mm F4.5-5.6 G SSM (SAL-70400G)', #17
+    54 => 'Carl Zeiss Vario-Sonnar T* 16-35mm F2.8 ZA SSM (SAL-1635Z)', #17
+    128 => 'Tamron Lens (128)',
+    128.1 => 'Tamron 18-200mm F3.5-6.3',
+    128.2 => 'Tamron 28-300mm F3.5-6.3',
+    128.3 => 'Tamron 80-300mm F3.5-6.3',
+    128.4 => 'Tamron AF 28-200mm F3.8-5.6 XR Di Aspherical [IF] MACRO', #JD
+    128.5 => 'Tamron SP AF 17-35mm F2.8-4 Di LD Aspherical IF', #JD
+    129 => 'Tamron Lens (129)',
+    129.1 => 'Tamron 200-400mm F5.6', #12
+    129.2 => 'Tamron 70-300mm F4-5.6 LD', #12
+    135 => 'Vivitar 28-210mm F3.5-5.6', #16
+    136 => 'Tokina EMZ M100 AF 100mm F3.5', #JD
+    137 => 'Cosina 70-210mm F2.8-4 AF', #11
     138 => 'Soligor 19-35mm F3.5-4.5', #11
+    142 => 'Voigtlander 70-300mm F4.5-5.6', #JD
+    146 => 'Voigtlander Macro APO-Lanthar 125mm F2.5 SL', #JD
+    255 => 'Tamron Lens (255)',
+    255.1 => 'Tamron SP AF 17-50mm F2.8 XR Di II LD Aspherical',
+    255.2 => 'Tamron AF 18-250mm F3.5-6.3 XR Di II LD', #JD
+    255.3 => 'Tamron AF 55-200mm F4-5.6 Di II',
+    255.4 => 'Tamron AF 70-300mm F4-5.6 Di LD MACRO 1:2',
+    255.5 => 'Tamron SP AF 200-500mm F5.0-6.3 Di LD IF',
     25501 => 'Minolta AF 50mm F1.7', #7
-    25511 => 'Minolta AF 35-70mm F4', # or Sigma UC AF 28-70mm F3.5-4.5 or Sigma M-AF 70-200mm F2.8 EX Aspherical', #/12/12
-    25521 => 'Minolta AF 28-85mm F3.5-4.5 [New]', # or Tokina 19-35mm F3.5-4.5 or Tokina 28-70mm F2.8 AT-X', #3/7
-    25531 => 'Minolta AF 28-135mm F4-4.5',
+    25511 => 'Minolta AF 35-70mm F4 or Other Lens',
+    25511.1 => 'Sigma UC AF 28-70mm F3.5-4.5', #12/16(HighSpeed-AF)
+    25511.2 => 'Sigma AF 28-70mm F2.8', #JD
+    25511.3 => 'Sigma M-AF 70-200mm F2.8 EX Aspherical', #12
+    25511.4 => 'Quantaray M-AF 35-80mm F4-5.6', #JD
+    25521 => 'Minolta AF 28-85mm F3.5-4.5 [New] or Other Lens',
+    25521.1 => 'Tokina 19-35mm F3.5-4.5', #3
+    25521.2 => 'Tokina 28-70mm F2.8 AT-X', #7
+    25521.3 => 'Tokina 80-400mm F4.5-5.6 AT-X AF II 840', #JD
+    25521.4 => 'Tokina AF PRO 28-80mm F2.8 AT-X 280', #JD
+    25521.5 => 'Tamron AF 19-35mm F3.5-4.5', #JD
+    25521.6 => 'Angenieux AF 28-70mm F2.6', #JD
+    25531 => 'Minolta AF 28-135mm F4-4.5 or Sigma Lens',
+    25531.1 => 'Sigma ZOOM-alpha 35-135mm F3.5-4.5', #16
+    25531.2 => 'Sigma 28-105mm F2.8-4 Aspherical', #JD
     25541 => 'Minolta AF 35-105mm F3.5-4.5', #13
-    25551 => 'Minolta AF 70-210mm F4 Macro', # or Sigma 70-210mm F4-5.6 APO or Sigma M-AF 70-200mm F2.8 EX APO', #7/6
+    25551 => 'Minolta AF 70-210mm F4 Macro or Sigma Lens',
+    25551.1 => 'Sigma 70-210mm F4-5.6 APO', #7
+    25551.2 => 'Sigma M-AF 70-200mm F2.8 EX APO', #6
     25561 => 'Minolta AF 135mm F2.8',
     25571 => 'Minolta AF 28mm F2.8',
     25581 => 'Minolta AF 24-50mm F4',
     25601 => 'Minolta AF 100-200mm F4.5',
-    25611 => 'Minolta AF 75-300mm F4.5-5.6', #13 # or Sigma 70-300mm F4-5.6 or Sigma 300mm F4 APO Macro', #12/3/7
-    25621 => 'Minolta AF 50mm F1.4 [New]', #13
-    25631 => 'Minolta AF 300mm F2.8 G',
+    25611 => 'Minolta AF 75-300mm F4.5-5.6 or Sigma Lens', #13
+    25611.1 => 'Sigma 70-300mm F4-5.6 DL Macro', #12
+    25611.2 => 'Sigma 300mm F4 APO Macro', #3/7
+    25611.3 => 'Sigma AF 500mm F4.5 APO', #JD
+    25611.4 => 'Sigma AF 170-500mm F5-6.3 APO Aspherical', #JD
+    25611.5 => 'Tokina AT-X AF 300mm F4', #JD
+    25611.6 => 'Tokina AF 730 II 75-300mm F4.5-5.6', #JD
+    25621 => 'Minolta/Sony AF 50mm F1.4 [New]', #13
+    25631 => 'Minolta AF 300mm F2.8 G or Sigma Lens',
+    25631.1 => 'Sigma AF 50-500mm F4-6.3 EX DG APO', #JD
+    25631.2 => 'Sigma AF 170-500mm F5-6.3 APO Aspherical', #JD
+    25631.3 => 'Sigma AF 500mm F4.5 EX DG APO', #JD
     25641 => 'Minolta AF 50mm F2.8 Macro',
     25651 => 'Minolta AF 600mm F4',
     25661 => 'Minolta AF 24mm F2.8',
-    25721 => 'Minolta AF 500mm F8 Reflex',
-    25781 => 'Minolta AF 16mm F2.8 Fisheye', # or Sigma 8mm F4 Fisheye or Sigma 14mm F3.5',
+    25721 => 'Minolta/Sony AF 500mm F8 Reflex',
+    25781 => 'Minolta AF 16mm F2.8 Fisheye or Sigma Lens',
+    25781.1 => 'Sigma 8mm F4 Fisheye',
+    25781.2 => 'Sigma 14mm F3.5',
+    25781.3 => 'Sigma 15mm F2.8 Fisheye', #JD (writes 16mm to EXIF)
     25791 => 'Minolta AF 20mm F2.8',
-    25811 => 'Minolta AF 100mm F2.8 Macro New', # or Tamron 90mm F2.8 Macro or Sigma 180mm F5.6 Macro',
-    25858 => 'Minolta AF 35-105mm F3.5-4.5 New', # or Tamron 24-135mm F3.5-5.6',
+    25811 => 'Minolta/Sony AF 100mm F2.8 Macro New or Sigma or Tamron Lens',
+    25811.1 => 'Sigma AF 90mm F2.8 Macro', #JD
+    25811.2 => 'Sigma AF 105mm F2.8 EX DG Macro', #JD
+    25811.3 => 'Sigma 180mm F5.6 Macro',
+    25811.4 => 'Tamron 90mm F2.8 Macro',
+    25851 => 'Beroflex 35-135mm F3.5-4.5', #16
+    25858 => 'Minolta AF 35-105mm F3.5-4.5 New or Tamron Lens',
+    25858.1 => 'Tamron 24-135mm F3.5-5.6',
     25881 => 'Minolta AF 70-210mm F3.5-4.5',
-    25891 => 'Minolta AF 80-200 F2.8 APO', # or Tokina 80-200mm F2.8',
+    25891 => 'Minolta AF 80-200mm F2.8 APO or Tokina Lens',
+    25891.1 => 'Tokina 80-200mm F2.8',
     25911 => 'Minolta AF 35mm F1.4', #(from Sony list)
     25921 => 'Minolta AF 85mm F1.4 G (D)',
     25931 => 'Minolta AF 200mm F2.8 G APO',
-    25941 => 'Minolta AF 3X-1X F1.7-2.8 Macro',
+    25941 => 'Minolta AF 3x-1x F1.7-2.8 Macro',
     25961 => 'Minolta AF 28mm F2',
     25971 => 'Minolta AF 35mm F2',
     25981 => 'Minolta AF 100mm F2',
@@ -130,7 +208,7 @@ $VERSION = '1.31';
     26241 => 'Minolta AF 35-80mm F4-5.6 Power Zoom',
     26281 => 'Minolta AF 80-200mm F2.8 G', #11
     26291 => 'Minolta AF 85mm F1.4 New',
-    26311 => 'Minolta AF 100-300mm F4.5-5.6 APO', #11
+    26311 => 'Minolta/Sony AF 100-300mm F4.5-5.6 APO', #11
     26321 => 'Minolta AF 24-50mm F4 New',
     26381 => 'Minolta AF 50mm F2.8 Macro New',
     26391 => 'Minolta AF 100mm F2.8 Macro',
@@ -141,7 +219,15 @@ $VERSION = '1.31';
     26671 => 'Minolta AF 35mm F2 New',
     26681 => 'Minolta AF 28mm F2 New',
     26721 => 'Minolta AF 24-105mm F3.5-4.5 (D)', #11
-    45741 => 'Minolta AF 200mm F2.8 G x2', # or Tokina 300mm F2.8 x2',
+    45741 => 'Minolta AF 200mm F2.8 G x2 or Tamron or Tokina Lens',
+    45741.1 => 'Tamron SP AF 90mm F2.5', #JD
+    45741.2 => 'Tokina RF 500mm F8.0 x2', #JD
+    45741.3 => 'Tokina 300mm F2.8 x2',
+    45851 => 'Tamron SP AF 300mm F2.8 LD IF', #11
+    65535 => 'T-Mount or Other Lens or no lens', #JD
+    65535.1 => 'Arax MC 35mm F2.8 Tilt+Shift', #JD
+    65535.2 => 'Arax MC 80mm F2.8 Tilt+Shift', #JD
+    65535.3 => 'Zenitar MF 16mm F2.8 Fisheye M42', #JD
 );
 
 %minoltaColorMode = (
@@ -161,6 +247,39 @@ $VERSION = '1.31';
     18 => 'Night Portrait', #10
 );
 
+%sonyColorMode = ( #15
+    0 => 'Standard',
+    1 => 'Vivid', #PH
+    2 => 'Portrait',
+    3 => 'Landscape',
+    4 => 'Sunset',
+    5 => 'Night Portrait',
+    6 => 'B&W',
+    7 => 'Adobe RGB',
+    12 => 'Neutral', #Sony4
+    100 => 'Neutral', #JD
+    101 => 'Clear', #JD
+    102 => 'Deep', #JD
+    103 => 'Light', #JD
+    104 => 'Night View', #JD
+    105 => 'Autumn Leaves', #JD
+);
+
+%minoltaSceneMode = (
+    0 => 'Standard',
+    1 => 'Portrait',
+    2 => 'Text',
+    3 => 'Night Scene',
+    4 => 'Sunset',
+    5 => 'Sports',
+    6 => 'Landscape',
+    7 => 'Night Portrait', #JD
+    8 => 'Macro',
+    9 => 'Super Macro',
+    16 => 'Auto',
+    17 => 'Night View/Portrait',
+);
+
 # Minolta tag table
 %Image::ExifTool::Minolta::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
@@ -182,7 +301,7 @@ $VERSION = '1.31';
     0x0003 => {
         Name => 'MinoltaCameraSettings',
         # These camera settings are different for the DiMAGE X31
-        Condition => '$self->{CameraModel} ne "DiMAGE X31"',
+        Condition => '$self->{Model} ne "DiMAGE X31"',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Minolta::CameraSettings',
             ByteOrder => 'BigEndian',
@@ -199,7 +318,7 @@ $VERSION = '1.31';
     # but it is an 8kB binary data block!
     0x0018 => {
         Name => 'ImageStabilization',
-        Condition => '$self->{CameraModel} =~ /^DiMAGE (A1|A2|X1)$/',
+        Condition => '$self->{Model} =~ /^DiMAGE (A1|A2|X1)$/',
         Notes => q{
             a block of binary data which exists in DiMAGE A2 (and A1/X1?) images only if
             image stabilization is enabled
@@ -211,9 +330,9 @@ $VERSION = '1.31';
         Writable => 'int32u',
     },
     0x0081 => {
-        # preview image in TIFF format files
+        # JPEG preview found in DiMAGE 7 images
         %Image::ExifTool::previewImageTagInfo,
-        Permanent => 1,     # don't add this to a file because it doesn't exist in JPEG images
+        Permanent => 1,     # don't add this to a file
     },
     0x0088 => {
         Name => 'PreviewImageStart',
@@ -230,23 +349,15 @@ $VERSION = '1.31';
         Writable => 'int32u',
         Protected => 2,
     },
-    0x0100 => { #10 (needs verification - PH)
+    0x0100 => { #10
         Name => 'SceneMode',
-        PrintConv => {
-            0 => 'Standard',
-            1 => 'Portrait',
-            2 => 'Text',
-            3 => 'Night Scene',
-            4 => 'Evening Scene',
-            5 => 'Sports',
-            6 => 'Landscape',
-            9 => 'Super Macro',
-        },
+        Writable => 'int32u',
+        PrintConv => \%minoltaSceneMode,
     },
     0x0101 => [
         {
             Name => 'ColorMode',
-            Condition => '$self->{CameraMake} !~ /^SONY/',
+            Condition => '$self->{Make} !~ /^SONY/',
             Priority => 0, # Other ColorMode is more reliable for A2
             Writable => 'int32u',
             PrintConv => \%minoltaColorMode,
@@ -254,17 +365,8 @@ $VERSION = '1.31';
         { #15
             Name => 'ColorMode',
             Writable => 'int32u',
-            Notes => 'Sony DSLR-A100',
-            PrintConv => {
-                0 => 'Standard',
-                1 => 'Vivid', #PH
-                2 => 'Portrait',
-                3 => 'Landscape',
-                4 => 'Sunset',
-                5 => 'Night Scene',
-                6 => 'B&W',
-                7 => 'Adobe RGB',
-            },
+            Notes => 'Sony models',
+            PrintConv => \%sonyColorMode,
         },
     ],
     0x0102 => {
@@ -287,7 +389,7 @@ $VERSION = '1.31';
         {
             Name => 'MinoltaQuality',
             Writable => 'int32u',
-            Condition => '$self->{CameraModel} =~ /^DiMAGE (A2|7Hi)$/',
+            Condition => '$self->{Model} =~ /^DiMAGE (A2|7Hi)$/',
             Notes => 'quality for DiMAGE A2/7Hi',
             Priority => 0, # lower priority because this doesn't work for A200
             PrintConv => { #4
@@ -302,7 +404,7 @@ $VERSION = '1.31';
         { #PH
             Name => 'MinoltaImageSize',
             Writable => 'int32u',
-            Condition => '$self->{CameraModel} !~ /^DiMAGE A200$/',
+            Condition => '$self->{Model} !~ /^DiMAGE A200$/',
             Notes => 'image size for other models except A200',
             PrintConv => {
                 1 => '1600x1200',
@@ -319,17 +421,18 @@ $VERSION = '1.31';
         Description => 'Flash Exposure Compensation',
         Writable => 'rational64s',
     },
-# needs verification:
-#    0x0105 => { #10
-#        Name => 'Teleconverter',
-#        PrintConv => {
-#            0 => 'None',
-#            72 => 'Minolta AF 2x APO (D)',
-#            80 => 'Minolta AF 2x APO II',
-#            136 => 'Minolta AF 1.4x APO (D)',
-#            144 => 'Minolta AF 1.4x APO II',
-#        },
-#    },
+    0x0105 => { #10
+        Name => 'Teleconverter',
+        Writable => 'int32u',
+        PrintHex => 1,
+        PrintConv => {
+            0x00 => 'None',
+            0x48 => 'Minolta AF 2x APO (D)',
+            0x50 => 'Minolta AF 2x APO II',
+            0x88 => 'Minolta AF 1.4x APO (D)',
+            0x90 => 'Minolta AF 1.4x APO II',
+        },
+    },
     0x0107 => { #8
         Name => 'ImageStabilization',
         Writable => 'int32u',
@@ -352,14 +455,19 @@ $VERSION = '1.31';
         Writable => 'int32u',
     },
     0x010c => { #3 (Alpha 7)
-        Name => 'LensID',
+        Name => 'LensType',
         Writable => 'int32u',
-        PrintConv => \%minoltaLensIDs,
+        Notes => q{
+            decimal values differentiate lenses which would otherwise have the same
+            LensType, and are used by the Composite LensID tag when attempting to
+            identify the specific lens model
+        },
+        PrintConv => \%minoltaLensTypes,
     },
     # 0x010e - WhiteBalance according to ref #10
     0x0113 => { #PH
         Name => 'ImageStabilization',
-        Condition => '$self->{CameraModel} eq "DSLR-A100"',
+        Condition => '$self->{Model} eq "DSLR-A100"',
         Notes => 'valid for Sony A100 only',
         Writable => 'int32u',
         PrintConv => { 0 => 'Off', 1 => 'On' },
@@ -367,7 +475,7 @@ $VERSION = '1.31';
     0x0114 => [
         { #8
             Name => 'MinoltaCameraSettings5D',
-            Condition => '$self->{CameraModel} =~ /^(DYNAX 5D|MAXXUM 5D|ALPHA SWEET)/',
+            Condition => '$self->{Model} =~ /^(DYNAX 5D|MAXXUM 5D|ALPHA SWEET)/',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Minolta::CameraSettings5D',
                 ByteOrder => 'BigEndian',
@@ -375,7 +483,7 @@ $VERSION = '1.31';
         },
         { #PH
             Name => 'MinoltaCameraSettingsA100',
-            Condition => '$self->{CameraModel} eq "DSLR-A100"',
+            Condition => '$self->{Model} eq "DSLR-A100"',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Minolta::CameraSettingsA100',
                 ByteOrder => 'BigEndian', # required because order differs for ARW and JPG images
@@ -393,6 +501,7 @@ $VERSION = '1.31';
     0x0f00 => {
         Name => 'MinoltaCameraSettings2',
         Writable => 0,
+        Binary => 1,
     },
 );
 
@@ -429,6 +538,7 @@ $VERSION = '1.31';
             1 => 'Red-eye reduction',
             2 => 'Rear flash sync',
             3 => 'Wireless',
+            4 => 'Off?', #PH
         },
     },
     3 => {
@@ -474,7 +584,7 @@ $VERSION = '1.31';
         Name => 'MeteringMode',
         PrintConv => {
             0 => 'Multi-segment',
-            1 => 'Center weighted',
+            1 => 'Center-weighted average',
             2 => 'Spot',
         },
     },
@@ -535,8 +645,8 @@ $VERSION = '1.31';
         Name => 'FocalLength',
         ValueConv => '$val / 256',
         ValueConvInv => '$val * 256',
-        PrintConv => 'sprintf("%.1fmm",$val)',
-        PrintConvInv => '$val=~s/mm$//;$val',
+        PrintConv => 'sprintf("%.1f mm",$val)',
+        PrintConvInv => '$val=~s/\s*mm$//;$val',
     },
     19 => {
         Name => 'FocusDistance',
@@ -598,15 +708,15 @@ $VERSION = '1.31';
     },
     31 => {
         Name => 'Saturation',
-        ValueConv => '$val - ($self->{CameraModel}=~/DiMAGE A2/ ? 5 : 3)',
-        ValueConvInv => '$val + ($self->{CameraModel}=~/DiMAGE A2/ ? 5 : 3)',
+        ValueConv => '$val - ($self->{Model}=~/DiMAGE A2/ ? 5 : 3)',
+        ValueConvInv => '$val + ($self->{Model}=~/DiMAGE A2/ ? 5 : 3)',
         PrintConv => 'Image::ExifTool::Exif::PrintParameter($val)',
         PrintConvInv => '$val=~/normal/i ? 0 : $val',
     },
     32 => {
         Name => 'Contrast',
-        ValueConv => '$val - ($self->{CameraModel}=~/DiMAGE A2/ ? 5 : 3)',
-        ValueConvInv => '$val + ($self->{CameraModel}=~/DiMAGE A2/ ? 5 : 3)',
+        ValueConv => '$val - ($self->{Model}=~/DiMAGE A2/ ? 5 : 3)',
+        ValueConvInv => '$val + ($self->{Model}=~/DiMAGE A2/ ? 5 : 3)',
         PrintConv => 'Image::ExifTool::Exif::PrintParameter($val)',
         PrintConvInv => '$val=~/normal/i ? 0 : $val',
     },
@@ -687,8 +797,8 @@ $VERSION = '1.31';
     },
     41 => {
         Name => 'ColorFilter',
-        ValueConv => '$val - ($self->{CameraModel}=~/DiMAGE A2/ ? 5 : 3)',
-        ValueConvInv => '$val + ($self->{CameraModel}=~/DiMAGE A2/ ? 5 : 3)',
+        ValueConv => '$val - ($self->{Model}=~/DiMAGE A2/ ? 5 : 3)',
+        ValueConvInv => '$val + ($self->{Model}=~/DiMAGE A2/ ? 5 : 3)',
     },
     42 => 'BWFilter',
     43 => {
@@ -741,7 +851,7 @@ $VERSION = '1.31';
     # 7Hi only:
     51 => {
         Name => 'ColorProfile',
-        Condition => '$self->{CameraModel} eq "DiMAGE 7Hi"',
+        Condition => '$self->{Model} eq "DiMAGE 7Hi"',
         Notes => 'DiMAGE 7Hi only',
         PrintConv => {
             0 => 'Not Embedded',
@@ -751,7 +861,7 @@ $VERSION = '1.31';
     # (the following may be entry 51 for other models?)
     52 => {
         Name => 'DataImprint',
-        Condition => '$self->{CameraModel} eq "DiMAGE 7Hi"',
+        Condition => '$self->{Model} eq "DiMAGE 7Hi"',
         Notes => 'DiMAGE 7Hi only',
         PrintConv => {
             0 => 'None',
@@ -765,7 +875,7 @@ $VERSION = '1.31';
         Name => 'FlashMetering',
         PrintConv => {
             0 => 'ADI (Advanced Distance Integration)',
-            1 => 'Pre-flash TTl',
+            1 => 'Pre-flash TTL',
             2 => 'Manual flash control',
         },
     },
@@ -788,7 +898,7 @@ $VERSION = '1.31';
             1 => 'Aperture Priority',
             2 => 'Shutter Priority',
             3 => 'Manual',
-            4 => 'Auto?',
+            4 => 'Auto',
             5 => 'Program-shift A',
             6 => 'Program-shift S',
         },
@@ -827,10 +937,11 @@ $VERSION = '1.31';
     0x0e => {
         Name => 'FocusMode',
         PrintConv => {
-            0 => 'Single-shot AF',
-            1 => 'Continuous AF',
-            3 => 'Automatic AF',
-            4 => 'Manual',
+            0 => 'AF-S',
+            1 => 'AF-C',
+            # Note: these two are reversed in ref 8
+            3 => 'Manual', #JD
+            4 => 'AF-A', #JD
         },
     },
     0x10 => {
@@ -851,15 +962,14 @@ $VERSION = '1.31';
         Name => 'Flash',
         PrintConv => { 0 => 'Off', 1 => 'On' },
     },
-# needs verification:
-#    0x16 => { #10
-#        Name => 'FlashMode',
-#        PrintConv => {
-#            0 => 'Normal',
-#            1 => 'Red-eye reduction',
-#            2 => 'Rear flash sync',
-#        },
-#    },
+    0x16 => { #10
+        Name => 'FlashMode',
+        PrintConv => {
+            0 => 'Normal',
+            1 => 'Red-eye reduction',
+            2 => 'Rear flash sync',
+        },
+    },
     0x1c => {
         Name => 'ISOSetting',
         PrintConv => {
@@ -910,13 +1020,11 @@ $VERSION = '1.31';
         ValueConv => '$val * 100',
         ValueConvInv => '$val / 100',
     },
-# needs verification:
-#    0x40 => { #10
-#        Name => 'HueAdjustment',
-#        Unknown => 1,
-#        ValueConv => '$val - 10',
-#        ValueConvInv => '$val + 10',
-#    },
+    0x40 => { #10
+        Name => 'HueAdjustment',
+        ValueConv => '$val - 10',
+        ValueConvInv => '$val + 10',
+    },
     0x46 => {
         Name => 'Rotation',
         PrintConv => {
@@ -1030,20 +1138,19 @@ $VERSION = '1.31';
             1 => 'Fired',
         },
     },
-# needs verification:
-#    0x20 => { #10
-#        Name => 'FlashMode',
-#        PrintConv => {
-#            0 => 'Normal',
-#            1 => 'Red-eye reduction',
-#            2 => 'Rear flash sync',
-#        },
-#    },
+    0x20 => { #10
+        Name => 'FlashMode',
+        PrintConv => {
+            0 => 'Normal',
+            1 => 'Red-eye reduction',
+            2 => 'Rear flash sync',
+        },
+    },
     0x25 => {
         Name => 'MeteringMode',
         PrintConv => {
             0 => 'Multi-segment',
-            1 => 'Center weighted',
+            1 => 'Center-weighted average',
             2 => 'Spot',
         },
     },
@@ -1067,17 +1174,16 @@ $VERSION = '1.31';
 #        ValueConv => '$val / 24',
 #        ValueConvInv => '$val * 24',
 #    },
-# needs verification:
-#    0x2f => { #10
-#        Name => 'ColorSpace',
-#        PrintConv => {
-#            0 => 'Natural sRGB',
-#            1 => 'Natural+ sRGB',
-#            2 => 'Monochrome',
-#            4 => 'Adobe RGB (ICC)',
-#            5 => 'Adobe RGB',
-#        },
-#    },
+    0x2f => { #10
+        Name => 'ColorSpace',
+        PrintConv => {
+            0 => 'Natural sRGB',
+            1 => 'Natural+ sRGB',
+            2 => 'Monochrome',
+            4 => 'Adobe RGB (ICC)',
+            5 => 'Adobe RGB',
+        },
+    },
     0x30 => {
         Name => 'Sharpness',
         ValueConv => '$val - 10',
@@ -1115,12 +1221,11 @@ $VERSION = '1.31';
         ValueConv => '$val * 100',
         ValueConvInv => '$val / 100',
     },
-# needs verification:
-#    0x4a => { #10
-#        Name => 'HueAdjustment',
-#        ValueConv => '$val - 10',
-#        ValueConvInv => '$val + 10',
-#    },
+    0x4a => { #10
+        Name => 'HueAdjustment',
+        ValueConv => '$val - 10',
+        ValueConvInv => '$val + 10',
+    },
     0x50 => {
         Name => 'Rotation',
         PrintConv => {
@@ -1137,15 +1242,14 @@ $VERSION = '1.31';
         PrintConvInv => 'eval $val',
     },
     0x54 => 'FreeMemoryCardImages',
-# needs verification:
-#    0x65 => { #10
-#        Name => 'Rotation',
-#        PrintConv => {
-#            0 => 'Horizontal (normal)',
-#            1=> 'Rotate 90 CW',
-#            2 => 'Rotate 270 CW',
-#        },
-#    },
+    0x65 => { #10
+        Name => 'Rotation',
+        PrintConv => {
+            0 => 'Horizontal (normal)',
+            1 => 'Rotate 90 CW',
+            2 => 'Rotate 270 CW',
+        },
+    },
     # 0x66 maybe program mode or some setting like this? (PH)
     0x6e => { #10
         Name => 'ColorTemperature',
@@ -1153,22 +1257,21 @@ $VERSION = '1.31';
         ValueConv => '$val * 100',
         ValueConvInv => '$val / 100',
     },
-# needs verification:
-#    0x71 => { #10
-#        Name => 'PictureFinish',
-#        PrintConv => {
-#            0 => 'Natural',
-#            1 => 'Natural+',
-#            2 => 'Portrait',
-#            3 => 'Wind Scene',
-#            4 => 'Evening Scene',
-#            5 => 'Night Scene',
-#            6 => 'Night Portrait',
-#            7 => 'Monochrome',
-#            8 => 'Adobe RGB',
-#            9 => 'Adobe RGB (ICC)',
-#        },
-#    },
+    0x71 => { #10
+        Name => 'PictureFinish',
+        PrintConv => {
+            0 => 'Natural',
+            1 => 'Natural+',
+            2 => 'Portrait',
+            3 => 'Wind Scene',
+            4 => 'Evening Scene',
+            5 => 'Night Scene',
+            6 => 'Night Portrait',
+            7 => 'Monochrome',
+            8 => 'Adobe RGB',
+            9 => 'Adobe RGB (ICC)',
+        },
+    },
     # 0x95 FlashStrength? (PH)
     # 0xa4 similar information to 0x27, except with different values
     0xae => {
@@ -1275,7 +1378,7 @@ $VERSION = '1.31';
         PrintConv => {
             0 => 'Single Frame',
             1 => 'Continuous',
-            2 => 'Self-Timer',
+            2 => 'Self-timer',
             3 => 'Continuous Bracketing',
             4 => 'Single-Frame Bracketing',
             5 => 'White Balance Bracketing',
@@ -1288,20 +1391,124 @@ $VERSION = '1.31';
     0x5a => { #15
         Name => 'Rotation',
         PrintConv => {
-            0 => 0,
-            1 => 270,
-            2 => 90,
+            0 => 'Horizontal (Normal)',
+            1 => 'Rotate 270 CW',
+            2 => 'Rotate 90 CW',
         },
     },
+    0x57 => { #15
+        Name => 'ImageStabilization',
+        Writable => 'int32u',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
     0x5e => { #15
         Name => 'ColorTemperature',
         ValueConv => '$val * 100',
         ValueConvInv => '$val / 100',
     },
-    0x87 => { #15
-        Name => 'ImageStabilization',
-        Writable => 'int32u',
-        PrintConv => { 0 => 'Off', 1 => 'On' },
+);
+
+# tags in Konica Minolta MOV videos (ref PH)
+# (similar information in Kodak,Minolta,Nikon,Olympus,Pentax and Sanyo videos)
+%Image::ExifTool::Minolta::MOV1 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FIRST_ENTRY => 0,
+    NOTES => q{
+        This information is found in MOV videos from some Konica Minolta models such
+        as the DiMage Z10 and X50.
+    },
+    0 => {
+        Name => 'Make',
+        Format => 'string[32]',
+    },
+    0x20 => {
+        Name => 'ModelType',
+        Format => 'string[8]',
+    },
+    # (01 00 at offset 0x28)
+    0x2e => {
+        Name => 'ExposureTime',
+        Format => 'int32u',
+        ValueConv => '$val ? 10 / $val : 0',
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+    },
+    0x32 => {
+        Name => 'FNumber',
+        Format => 'rational64u',
+        PrintConv => 'sprintf("%.1f",$val)',
+    },
+    0x3a => {
+        Name => 'ExposureCompensation',
+        Format => 'rational64s',
+        PrintConv => 'Image::ExifTool::Exif::ConvertFraction($val)',
+    },
+    # 0x4c => 'WhiteBalance', ?
+    0x50 => {
+        Name => 'FocalLength',
+        Format => 'rational64u',
+        PrintConv => 'sprintf("%.1f mm",$val)',
+    },
+);
+
+# tags in Minolta MOV videos (ref PH)
+# (similar information in Kodak,Minolta,Nikon,Olympus,Pentax and Sanyo videos)
+%Image::ExifTool::Minolta::MOV2 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FIRST_ENTRY => 0,
+    NOTES => q{
+        This information is found in MOV videos from some Minolta models such as the
+        DiMAGE X and Xt.
+    },
+    0 => {
+        Name => 'Make',
+        Format => 'string[32]',
+    },
+    0x18 => {
+        Name => 'ModelType',
+        Format => 'string[8]',
+    },
+    # (01 00 at offset 0x20)
+    0x26 => {
+        Name => 'ExposureTime',
+        Format => 'int32u',
+        ValueConv => '$val ? 10 / $val : 0',
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+    },
+    0x2a => {
+        Name => 'FNumber',
+        Format => 'rational64u',
+        PrintConv => 'sprintf("%.1f",$val)',
+    },
+    0x32 => {
+        Name => 'ExposureCompensation',
+        Format => 'rational64s',
+        PrintConv => 'Image::ExifTool::Exif::ConvertFraction($val)',
+    },
+    # 0x44 => 'WhiteBalance', ?
+    0x48 => {
+        Name => 'FocalLength',
+        Format => 'rational64u',
+        PrintConv => 'sprintf("%.1f mm",$val)',
+    },
+);
+
+# more tags in Minolta MOV videos (ref PH)
+%Image::ExifTool::Minolta::MMA = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    NOTES => q{
+        This information is found in MOV videos from Minolta models such as the
+        DiMAGE A2, S414 and 7Hi.
+    },
+    0 => {
+        Name => 'Make',
+        Format => 'string[20]',
+    },
+    20 => {
+        Name => 'SoftwareVersion',
+        Format => 'string[16]',
     },
 );
 
@@ -1336,13 +1543,18 @@ sub ConvertWhiteBalance($)
     my $val = shift;
     my $printConv = $minoltaWhiteBalance{$val};
     unless (defined $printConv) {
-        # the A2 values can be shifted by += 3 settings, where
-        # each setting adds or subtracts 0x001000 (ref 2)
-        my $type = ($val & 0xff000000) + 0x800000;
-        if ($type and $printConv = $minoltaWhiteBalance{$type}) {
-            $printConv .= sprintf("%+.8g", ($val - $type) / 0x10000);
+        if ($val & 0xffff0000) {
+            # the A2 values can be shifted by += 3 settings, where
+            # each setting adds or subtracts 0x0010000 (ref 2)
+            my $type = ($val & 0xff000000) + 0x800000;
+            if ($minoltaWhiteBalance{$type}) {
+                $printConv = $minoltaWhiteBalance{$type} .
+                             sprintf("%+.8g", ($val - $type) / 0x10000);
+            } else {
+                $printConv = sprintf("Unknown (0x%x)", $val);
+            }
         } else {
-            $printConv = sprintf("Unknown (0x%x)", $val);
+            $printConv = sprintf("Unknown ($val)");
         }
     }
     return $printConv;
@@ -1368,7 +1580,7 @@ and write Minolta RAW (MRW) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -1386,7 +1598,8 @@ under the same terms as Perl itself.
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to Jay Al-Saadi, Niels Kristian Bech Jensen, Shingo Noguchi, Pedro
-Corte-Real and Jeffery Small for the information they provided.
+Corte-Real, Jeffery Small, Jens Duttke,  Thomas Kassner and Mladen Sever for
+the information they provided.
 
 =head1 SEE ALSO
 
