@@ -22,7 +22,7 @@ my $debug = 0;
 my $numPadLines = 24;       # number of blank padding lines
 
 # when writing extended XMP, resources bigger than this get placed in their own
-# rdf:Description so they can be moved to the extended segements if necessary
+# rdf:Description so they can be moved to the extended segments if necessary
 my $newDescThresh = 10000;  # 10 kB
 
 # individual resources and namespaces to place last inseparate rdf:Description's
@@ -337,6 +337,19 @@ my %xmpStruct = (
         Sublocation  => { },
         WorldRegion  => { },
     },
+    # Microsoft Photo 1.2 structures
+    RegionInfo => {
+        NAMESPACE => 'MPRI',
+        Regions => {
+            Struct => 'Regions',
+            List => 'Bag',
+        },
+    },
+    Regions => {
+        NAMESPACE => 'MPReg',
+        Rectangle => { },
+        PersonDisplayName => { },
+    },
 );
 
 my $rdfDesc = 'rdf:Description';
@@ -379,7 +392,6 @@ my $pktCloseR =  "<?xpacket end='r'?>";
     }
     # update XMP tag tables
     my $mainTable = GetTagTable('Image::ExifTool::XMP::Main');
-    GenerateTagIDs($mainTable);
     foreach $mainTag (keys %$mainTable) {
         my $mainInfo = $mainTable->{$mainTag};
         next unless ref $mainInfo eq 'HASH' and $mainInfo->{SubDirectory};
@@ -398,7 +410,6 @@ my $pktCloseR =  "<?xpacket end='r'?>";
             $$table{NAMESPACE} = $ns;
         }
         $$table{WRITE_PROC} = \&WriteXMP;   # set WRITE_PROC for all tables
-        GenerateTagIDs($table);
         $table->{CHECK_PROC} = \&CheckXMP;  # add our write check routine
         foreach $tag (TagTableKeys($table)) {
             my $tagInfo = $$table{$tag};
@@ -422,7 +433,7 @@ sub XMPOpen($)
     my $tk;
     if (defined $nv) {
         $tk = Image::ExifTool::GetNewValues($nv);
-        $exifTool->VPrint(1, $tk ? "    + XMP-x:XMPToolkit = '$tk'\n" : "    - XMP-x:XMPToolkit\n");
+        $exifTool->VerboseValue(($tk ? '+' : '-') . ' XMP-x:XMPToolkit', $tk);
         ++$exifTool->{CHANGED};
     } else {
         $tk = "Image::ExifTool $Image::ExifTool::VERSION";
@@ -487,15 +498,27 @@ sub FormatXMPDate($)
 sub CheckXMP($$$)
 {
     my ($exifTool, $tagInfo, $valPtr) = @_;
-    # convert value from Latin if necessary
-    if ($exifTool->{OPTIONS}->{Charset} eq 'Latin' and $$valPtr =~ /[\x80-\xff]/) {
-        # convert from Latin to UTF-8
-        my $val = Image::ExifTool::Latin2Unicode($$valPtr,'n');
-        $$valPtr = Image::ExifTool::Unicode2UTF8($val,'n');
-    }
     my $format = $tagInfo->{Writable};
-    # if no format specified, value is a simple string
-    return undef unless $format and $format ne 'string';
+    # (if no format specified, value is a simple string)
+    if (not $format or $format eq 'string' or $format eq 'lang-alt') {
+        # convert value from Latin if necessary
+        if ($exifTool->{OPTIONS}->{Charset} eq 'Latin') {
+            if ($$valPtr =~ /[\x80-\xff]/) {
+                # convert from Latin to UTF-8
+                my $val = Image::ExifTool::Latin2Unicode($$valPtr,'n');
+                $$valPtr = Image::ExifTool::Unicode2UTF8($val,'n');
+            }
+        } else {
+            # translate invalid XML characters to "."
+            $$valPtr =~ tr/\0-\x08\x0b\x0c\x0e-\x1f/./;
+            # fix any malformed UTF-8 characters
+            if (FixUTF8($valPtr) and not $$exifTool{WarnBadUTF8}) {
+                $exifTool->Warn('Malformed UTF-8 character(s)');
+                $$exifTool{WarnBadUTF8} = 1;
+            }
+        }
+        return undef;   # success
+    }
     if ($format eq 'rational' or $format eq 'real') {
         # make sure the value is a valid floating point number
         Image::ExifTool::IsFloat($$valPtr) or return 'Not a floating point number';
@@ -515,8 +538,6 @@ sub CheckXMP($$$)
         my $newDate = FormatXMPDate($$valPtr);
         return "Invalid date/time (use YYYY:MM:DD HH:MM:SS[.SS][+/-HH:MM|Z])" unless $newDate;
         $$valPtr = $newDate;
-    } elsif ($format eq 'lang-alt') {
-        # nothing to do
     } elsif ($format eq 'boolean') {
         if (not $$valPtr or $$valPtr =~ /false/i or $$valPtr =~ /^no$/i) {
             $$valPtr = 'False';
@@ -568,17 +589,17 @@ sub SetPropertyPath($$;$$)
         $propList[-1] =~ s/-$$tagInfo{LangCode}$// if $$tagInfo{LangCode};
         # handle lists of lang-alt lists (ie. XMP-plus:Custom tags)
         if ($$tagInfo{List} and $$tagInfo{List} ne '1') {
-            push @propList, "rdf:$$tagInfo{List}", 'rdf:li 000';
+            push @propList, "rdf:$$tagInfo{List}", 'rdf:li 10';
         }
     } else {
         $listType = $$tagInfo{List};
     }
     # add required properties if this is a list
-    push @propList, "rdf:$listType", 'rdf:li 000' if $listType and $listType ne '1';
+    push @propList, "rdf:$listType", 'rdf:li 10' if $listType and $listType ne '1';
     # set PropertyPath for all elements of this structure if necessary
     my $structName = $$tagInfo{Struct};
     if ($structName) {
-        my $struct = $xmpStruct{$structName} || 
+        my $struct = $xmpStruct{$structName} ||
                      $Image::ExifTool::UserDefined::xmpStruct{$structName};
         $struct or warn("No XMP $$tagInfo{Struct} structure!\n"), return;
         my $tag;
@@ -600,7 +621,7 @@ sub SetPropertyPath($$;$$)
         } else {
             $listType = $$tagInfo{List};
         }
-        push @propList, "rdf:$listType", 'rdf:li 000' if $listType and $listType ne '1';
+        push @propList, "rdf:$listType", 'rdf:li 10' if $listType and $listType ne '1';
     }
     # set property path for tagInfo in main table
     $$tagInfo{PropertyPath} = join '/', @propList;
@@ -621,6 +642,11 @@ sub CaptureXMP($$$;$)
     {
         # no properties to save yet if this is just the description
         return unless @$propList > 3;
+        # ignore empty list properties
+        if ($$propList[-1] =~ /^rdf:(Bag|Seq|Alt)$/) {
+            $exifTool->Warn("Ignored empty $$propList[-1] list for $$propList[-2]", 1);
+            return;
+        }
         # save information about this property
         my $capture = $exifTool->{XMP_CAPTURE};
         my $path = join('/', @$propList[3..$#$propList]);
@@ -635,7 +661,7 @@ sub CaptureXMP($$$;$)
         # set flag so we don't write x:xmpmeta element
         $exifTool->{XMP_NO_XMPMETA} = 1;
         # add missing x:xmpmeta element and try again
-        unshift @$propList, 'x:xmpmeta';    
+        unshift @$propList, 'x:xmpmeta';
         CaptureXMP($exifTool, $propList, $val, $attrs);
     } else {
         $exifTool->{XMP_ERROR} = 'Improperly enclosed XMP property: ' . join('/',@$propList);
@@ -844,7 +870,7 @@ sub LimitXMPSize($$$$$$)
         $guid = uc unpack('H*', Digest::MD5::md5($extData));
         $newData =~ s/0{32}/$guid/;     # update GUID in main XMP segment
     }
-    $exifTool->VPrint(1, "    + XMP-xmpNote:HasExtendedXMP = '$guid'\n");
+    $exifTool->VerboseValue('+ XMP-xmpNote:HasExtendedXMP', $guid);
     $$dataPt = $newData;        # return main XMP block
     return (\$extData, $guid);  # return extended XMP and its GUID
 }
@@ -919,8 +945,8 @@ sub WriteXMP($$;$)
             $about = Image::ExifTool::GetNewValues($exifTool->{NEW_VALUE}->{$tagInfo}) || '';
             if ($verbose > 1) {
                 my $wasAbout = $exifTool->{XMP_ABOUT};
-                $exifTool->VPrint(1, "    - XMP-rdf:About = '", UnescapeXML($wasAbout), "'\n") if defined $wasAbout;
-                $exifTool->VPrint(1, "    + XMP-rdf:About = '$about'\n");
+                $exifTool->VerboseValue('- XMP-rdf:About', UnescapeXML($wasAbout)) if defined $wasAbout;
+                $exifTool->VerboseValue('+ XMP-rdf:About', $about);
             }
             $about = EscapeXML($about); # must escape for XML
             ++$changed;
@@ -944,8 +970,6 @@ sub WriteXMP($$;$)
                 $exifTool->VPrint(0, "  Writing XMP as a block\n");
                 ++$exifTool->{CHANGED};
                 Write($$dirInfo{OutFile}, $newVal) or $rtnVal = -1;
-            } else {
-                $exifTool->Error("Can't delete all XMP from an XMP file");
             }
             delete $exifTool->{XMP_CAPTURE};
             return $rtnVal;
@@ -982,10 +1006,7 @@ sub WriteXMP($$;$)
                 my $ucg = uc $grp;
                 next unless $$del{$ucg} or ($$del{'XMP-*'} and not $$del{"-$ucg"});
             }
-            if ($verbose > 1) {
-                my $val = $capture{$path}->[0];
-                $exifTool->VPrint(1, "    - $grp:$tag = '$val'\n");
-            }
+            $exifTool->VerboseValue("- $grp:$tag", $capture{$path}->[0]);
             delete $capture{$path};
             ++$changed;
         }
@@ -993,10 +1014,7 @@ sub WriteXMP($$;$)
     # delete HasExtendedXMP tag (we create it as needed)
     my $hasExtTag = 'xmpNote:HasExtendedXMP';
     if ($capture{$hasExtTag}) {
-        if ($verbose > 1) {
-            my $val = $capture{$hasExtTag}->[0];
-            $exifTool->VPrint(1, "    - XMP-$hasExtTag = '$val'\n");
-        }
+        $exifTool->VerboseValue("- XMP-$hasExtTag", $capture{$hasExtTag}->[0]);
         delete $capture{$hasExtTag};
     }
     # set $xmpOpen now to to handle xmptk tag first
@@ -1030,8 +1048,8 @@ sub WriteXMP($$;$)
             my ($path2) = grep /^\Q$path\E$/i, keys %capture;
             $path2 and $capList = $capture{$path = $path2};
         }
-        my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
-        my $overwrite = Image::ExifTool::IsOverwriting($newValueHash);
+        my $nvHash = $exifTool->GetNewValueHash($tagInfo);
+        my $overwrite = Image::ExifTool::IsOverwriting($nvHash);
         my $writable = $$tagInfo{Writable} || '';
         my (%attrs, $deleted, $added);
         # delete existing entry if necessary
@@ -1043,7 +1061,7 @@ sub WriteXMP($$;$)
                 # check to see if this is an indexed list item
                 if ($path =~ / /) {
                     my $pathPattern;
-                    ($pathPattern = $path) =~ s/ 000/ \\d\{3\}/g;
+                    ($pathPattern = $path) =~ s/ \d+/ \\d\+/g;
                     @matchingPaths = sort grep(/^$pathPattern$/, keys %capture);
                 } else {
                     push @matchingPaths, $path;
@@ -1052,14 +1070,14 @@ sub WriteXMP($$;$)
                     my ($val, $attrs) = @{$capture{$path}};
                     if ($overwrite < 0) {
                         # only overwrite specific values
-                        next unless Image::ExifTool::IsOverwriting($newValueHash, UnescapeXML($val));
+                        next unless Image::ExifTool::IsOverwriting($nvHash, UnescapeXML($val));
                     }
                     if ($writable eq 'lang-alt') {
                         # get original language code (lc for comparisons)
                         $oldLang = lc($$attrs{'xml:lang'} || 'x-default');
                         # delete all if deleting "x-default" or writing with no LangCode
                         # (XMP spec requires x-default language exist and be first in list)
-                        if ($oldLang eq 'x-default' and not ($newValueHash->{Value} or
+                        if ($oldLang eq 'x-default' and not ($nvHash->{Value} or
                             ($$tagInfo{LangCode} and $$tagInfo{LangCode} ne 'x-default')))
                         {
                             $delLang = 1;   # delete all languages
@@ -1075,7 +1093,7 @@ sub WriteXMP($$;$)
                         my $tagName = $$tagInfo{Name};
                         $tagName =~ s/-$$tagInfo{LangCode}$// if $$tagInfo{LangCode};
                         $tagName .= '-' . $$attrs{'xml:lang'} if $$attrs{'xml:lang'};
-                        $exifTool->VPrint(1, "    - $grp:$tagName = '$val'\n");
+                        $exifTool->VerboseValue("- $grp:$tagName", $val);
                     }
                     # save attributes and path from first deleted property
                     # so we can replace it exactly
@@ -1102,26 +1120,31 @@ sub WriteXMP($$;$)
                     # unless this is a list or an lang-alt tag
                     next unless $$tagInfo{List} or $oldLang;
                     # (match last index to put in same lang-alt list for Bag of lang-alt items)
-                    $path =~ m/.* (\d{3})/g or warn "Internal error: no list index!\n", next;
+                    $path =~ m/.* (\d+)/g or warn "Internal error: no list index!\n", next;
                     $added = $1;
                 }
-            } elsif ($path =~ m/.* (\d{3})/g) {  # (match last index)
+            } elsif ($path =~ m/.* (\d+)/g) {  # (match last index)
                 $added = $1;
             }
             if (defined $added) {
                 # add to end of list
-                my $pos = pos($path) - 3;
+                my $len = length $added;
+                my $pos = pos($path) - $len;
+                my $nxt = substr($added, 1) + 1;
                 for (;;) {
-                    substr($path, $pos, 3) = ++$added;
+                    my $try = length($nxt) . $nxt;
+                    substr($path, $pos, $len) = $try;
                     last unless $capture{$path};
+                    $len = length $try;
+                    ++$nxt;
                 }
             }
         }
         # check to see if we want to create this tag
         # (create non-avoided tags in XMP data files by default)
-        my $isCreating = (Image::ExifTool::IsCreating($newValueHash) or
+        my $isCreating = (Image::ExifTool::IsCreating($nvHash) or
                           ($preferred and not $$tagInfo{Avoid} and
-                            not defined $$newValueHash{Shift}));
+                            not defined $$nvHash{Shift}));
 
         # don't add new values unless...
             # ...tag existed before and was deleted, or we added it to a list
@@ -1130,7 +1153,7 @@ sub WriteXMP($$;$)
             (not $capList and $isCreating);
 
         # get list of new values (all done if no new values specified)
-        my @newValues = Image::ExifTool::GetNewValues($newValueHash) or next;
+        my @newValues = Image::ExifTool::GetNewValues($nvHash) or next;
 
         # set language attribute for lang-alt lists
         if ($writable eq 'lang-alt') {
@@ -1143,23 +1166,23 @@ sub WriteXMP($$;$)
                     my $tagName = $$tagInfo{Name};
                     $tagName =~ s/-$$tagInfo{LangCode}$/-x-default/;
                     my $grp = $exifTool->GetGroup($tagInfo, 1);
-                    $exifTool->VPrint(1, "    + $grp:$tagName = '$newValue'\n");
+                    $exifTool->VerboseValue("+ $grp:$tagName", $newValue);
                 }
-                $path =~ s/(.*) 000/$1 001/ or warn "Internal error: no list index!\n", next;
+                $path =~ s/(.*) 10/$1 11/ or warn "Internal error: no list index!\n", next;
             }
         }
-        
+
         # add new value(s) to %capture hash
         for (;;) {
             my $newValue = EscapeXML(shift @newValues);
             if ($$tagInfo{Resource}) {
-                $capture{$path} = [ '', { %attrs, 'rdf:resource' => $newValue } ]; 
+                $capture{$path} = [ '', { %attrs, 'rdf:resource' => $newValue } ];
             } else {
                 $capture{$path} = [ $newValue, \%attrs ];
             }
             if ($verbose > 1) {
                 my $grp = $exifTool->GetGroup($tagInfo, 1);
-                $exifTool->VPrint(1, "    + $grp:$$tagInfo{Name} = '$newValue'\n");
+                $exifTool->VerboseValue("+ $grp:$$tagInfo{Name}", $newValue);
             }
             ++$changed;
             # add rdf:type if necessary
@@ -1168,14 +1191,17 @@ sub WriteXMP($$;$)
             }
             last unless @newValues;
             # (match first index to put in different lang-alt list for Bag of lang-alt items)
-            $path =~ m/ (\d{3})/g or warn("Internal error: no list index!\n"), next;
-            my $listIndex = $1;
-            my $pos = pos($path) - 3;
+            $path =~ m/ (\d+)/g or warn("Internal error: no list index!\n"), next;
+            my $len = length $1;
+            my $pos = pos($path) - $len;
+            my $nxt = substr($1, 1) + 1;
             for (;;) {
-                substr($path, $pos, 3) = ++$listIndex;
+                my $try = length($nxt) . $nxt;
+                substr($path, $pos, $len) = $try;
                 last unless $capture{$path};
+                $len = length $try;
+                ++$nxt;
             }
-            $capture{$path} and warn("Too many entries in XMP list!\n"), next;
         }
     }
     # remove the ExifTool members we created
@@ -1188,11 +1214,7 @@ sub WriteXMP($$;$)
         return undef unless $xmpFile;   # just rewrite original XMP
         # get DataPt again because it may have been set by ProcessXMP
         $dataPt = $$dirInfo{DataPt};
-        unless (defined $dataPt) {
-            $exifTool->Error("Nothing to write");
-            return 1;
-        }
-        Write($$dirInfo{OutFile}, $$dataPt) or return -1;
+        Write($$dirInfo{OutFile}, $$dataPt) or return -1 if defined $dataPt;
         return 1;
     }
 #
@@ -1213,7 +1235,6 @@ sub WriteXMP($$;$)
     my (@curPropList, @writeLast, @descStart, $extStart);
     my (%nsCur, $prop, $n, $lastDesc, $path);
     my @pathList = sort TypeFirst keys %capture;
-
     # order properties to write large values last if we have a MaxDataLen limit
     if ($maxDataLen and @pathList) {
         my @pathTmp;
@@ -1440,7 +1461,7 @@ This file contains routines to write XMP metadata.
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
